"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const lodash_1 = __importDefault(require("lodash"));
const request_promise_1 = __importDefault(require("request-promise"));
const https_1 = require("https");
// tslint:disable-next-line: no-var-requires
const CFError = require('cf-errors');
const LIMIT_PER_PAGE = 500;
const MAX_PAGES = 20;
const ApiVersions = {
    V1: 'rest/api/1.0/',
};
const _cleanEncodedFilePath = (filepath) => {
    const prefix = './';
    const hasPrefix = filepath.indexOf(prefix) === 0;
    const path = hasPrefix ? filepath.substring(prefix.length) : filepath;
    return encodeURIComponent(path);
};
class BitbucketServer {
    constructor(opts) {
        if (!opts.apiURL) {
            throw new CFError('Cannot create bitbucket-server provider without api url');
        }
        this.baseUrl = opts.apiURL;
        if (!this.baseUrl.endsWith('/')) {
            this.baseUrl = `${this.baseUrl}/`;
        }
        this.timeout = opts.timeout || 10000;
        if (opts.type === 'oauth2') {
            this.authenticationHeader = { Authorization: `Bearer ${opts.password}` };
        }
        else {
            this.authenticationHeader = { Authorization: `Basic ${Buffer.from(`${opts.username}:${opts.password}`).toString('base64')}` };
        }
        if (process.env.NODE_TLS_REJECT_UNAUTHORIZED === '0' || opts.insecure) {
            this.agent = new https_1.Agent({ rejectUnauthorized: false });
        }
    }
    async performAPICall(opts) {
        const method = opts.method || 'GET';
        const requestHeaders = lodash_1.default.merge(opts.noAuth ? {} : this.authenticationHeader, opts.headers || {});
        const requestOptions = {
            method,
            url: `${this.baseUrl}${opts.apiVersion}${opts.api}`,
            qs: opts.qs || {},
            headers: requestHeaders,
            json: opts.json,
            timeout: this.timeout,
            form: opts.data,
            resolveWithFullResponse: true,
            agent: this.agent,
        };
        return request_promise_1.default(requestOptions);
    }
    getName() {
        return 'bitbucket-server';
    }
    async fetchRawFile(opts) {
        const lines = [];
        let start = 0;
        let page = 0;
        let isLastPage = false;
        while (!isLastPage && page < MAX_PAGES) {
            const res = await this.performAPICall({
                apiVersion: ApiVersions.V1,
                api: `projects/${opts.owner}/repos/${opts.repo}/browse/${_cleanEncodedFilePath(opts.path)}`,
                json: true,
                qs: {
                    at: opts.ref,
                    raw: 'true',
                    start: '' + start,
                    limit: '' + LIMIT_PER_PAGE,
                }
            });
            if (res.statusCode >= 400) {
                throw new CFError(`Failed to retrieve file: ${JSON.stringify(opts)}, status code: ${res.statusCode}`);
            }
            isLastPage = res.body.isLastPage;
            lines.push(...lodash_1.default.map(res.body.lines, (line) => line.text));
            start += LIMIT_PER_PAGE;
            page += 1;
        }
        return lines.join('\n');
    }
    async getBranch(opts) {
        const commitMetadataPath = 'com.atlassian.bitbucket.server.bitbucket-branch:latest-commit-metadata';
        let branch;
        let start = 0;
        let page = 0;
        let isLastPage = false;
        while (!isLastPage && page < MAX_PAGES) {
            const res = await this.performAPICall({
                apiVersion: ApiVersions.V1,
                api: `projects/${opts.owner}/repos/${opts.repo}/branches`,
                json: true,
                qs: {
                    filterText: opts.branch,
                    details: 'true',
                    start: '' + start,
                    limit: '' + LIMIT_PER_PAGE,
                }
            });
            if (res.statusCode >= 400) {
                throw new CFError(`Failed to retrieve branch: ${JSON.stringify(opts)}, status code: ${res.statusCode}`);
            }
            isLastPage = res.body.isLastPage;
            branch = lodash_1.default.find(res.body.values, (b) => lodash_1.default.isEqual(opts.branch, b.displayId ? b.displayId : lodash_1.default.last(b.id.split('/'))));
            if (branch) {
                break; // found the correct branch
            }
            start += LIMIT_PER_PAGE;
            page += 1;
        }
        if (!branch) {
            throw new CFError(`Failed to retrieve branch: ${JSON.stringify(opts)}, branch not found`);
        }
        return {
            name: branch.displayId,
            id: branch.id,
            commit: {
                sha: branch.metadata[commitMetadataPath].id,
                commiter_name: branch.metadata[commitMetadataPath].author.name,
                message: branch.metadata[commitMetadataPath].message,
                url: '',
            }
        };
    }
    async getRepository(opt) {
        throw new Error("Method not implemented.");
    }
    async listBranches(opt) {
        throw new Error("Method not implemented.");
    }
    async listRepositoriesForOwner(opt) {
        throw new Error("Method not implemented.");
    }
    async createRepositoryWebhook(opt) {
        throw new Error("Method not implemented.");
    }
    async listWebhooks(opt) {
        throw new Error("Method not implemented.");
    }
    async deleteRepositoryWebhook(opt) {
        throw new Error("Method not implemented.");
    }
    async listRepositoriesWithAffiliation(opt) {
        throw new Error("Method not implemented.");
    }
    async listOrganizations(opt) {
        throw new Error("Method not implemented.");
    }
    async listRepositoriesForOrganization(opt) {
        throw new Error("Method not implemented.");
    }
}
exports.default = BitbucketServer;
//# sourceMappingURL=bitbucket-server.js.map